;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------

;;
;; ENTITY AI-CONTROL SYSTEM
;;
.include "entity_manager.h.s"
.include "cpctelera.h.s"

.module ai_control_system

.globl _sp_enemigo
;.globl _sp_fondoAzul
;;==============================================================================================
;; AI Control System Constants

timeStartChangeStatus = 50
_timeChangeStatus: .db #timeStartChangeStatus
ai_control_update_time:
    ld      a, (_timeChangeStatus)
    cp      a, #0
    jr      z, _changeStatus
    ;;Timer not 0
        dec     a
        ld      (_timeChangeStatus), a
    jr      _endStatus
_changeStatus:
    ;;Timer 0
    ld      e_ai_st(ix), #e_ai_st_stand_by
    ld      a, #timeStartChangeStatus
    ld      (_timeChangeStatus), a
_endStatus:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Init
;;  Inits the AI control system
;; INPUT:
;;     HL = Pointer to the entity array
;;
ai_control_sys_init::
    ld      (_ent_array_ptr_temp_standby), hl   ;; [6]

    ld      (_ent_array_ptr), hl                ;; [6]

    ld      (_ent_array_ptr_reviveEnemy), hl
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Update
;;   Update all AI entities
;; INPUT:
;;     IX = Pointer to the entity array
;;      A = Number of elements in the array
;; DESTROYS: AF, BC, DE, IX
;; STACK USE: 2 bytes
;;
ai_control_sys_update::
;    ld      (_ent_counter), a               ;; guardo en el contador el maximo numero de entidades que existe

_ent_array_ptr = . + 2
    ld      ix, #0x0000                     ;; Pointer to entity Array, needed later

_loop:

    ld      a, e_w(ix)                      ;; A = Entity.witdh
    cp      #e_w_invalid                    ;; if (Entity.witdh == invalid)
    ret     z                               ;; return

    ld      a, e_cmp_type(ix)               ;;
    cp      #e_t_invalid                    ;; FUNCIONA :D
    jr      z, _no_AI_ent                   ;;

    ld      a, e_ai_st(ix)                  ;; Load entity state
    cp      #e_ai_st_noAI                   ;; Compare with no AI
    jr      z, _no_AI_ent                   ;;  if = 0, jump to _no_AI_ent
_AI_ent:                                    ;; Si no da zero entro y modifico su velocidad
    cp      #e_ai_st_stand_by               ;; Check if stand_by is zero
    call    z, sys_ai_stand_by              ;; if zero, call to sys_ai_stand_by function
    cp      #e_ai_st_move_to                ;; Check if move_to is zero
    call    z, sys_ai_move_to               ;; if, zero, call to sys_ai_move_to function

_no_AI_ent:

    ;; Jump to next entity
    ld      de, #e_sizeof
    add     ix, de

jr      _loop



;;==============================================================================================
;;==============================================================================================
;; BEHAVIOURS
;;==============================================================================================
;;==============================================================================================


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Stand_by
;;   An entity is waiting for an event to occur
;; INPUT:
;;     IX = Pointer to the entity
;; DESTROYS: AF, IY
;;
sys_ai_stand_by::
_ent_array_ptr_temp_standby = .+2           ;; Pointer to entity array, which is equivalent to habing a pointer to the fisrt entity
    ld      iy, #0x0000                     ;; iy points to player

    ld      e_ai_aim_x(iy), #1
    ld      a, e_ai_aim_x(iy)               ;; Load the value Cargo este valor que es el target del pleyer, que no se usa aun porque no hace move_to
    or      a                               ;; Or a with a, changing F
    ret     z                               ;; if zero, player did not move

;; Pressed key Space. Move to player
    ld      a, e_x(iy)
    ld      e_ai_aim_x(ix), a               ;; En e_ai_aim_x esta la tecla de espacio en 1 cuando la pulsa
    ld      a, e_y(iy)
    ld      e_ai_aim_y(ix), a
    ;; New status               = move_to
    ld      e_ai_st(ix), #e_ai_st_move_to
ret


;;===================================================================================================
;;
;;NOTE:     Player has pair velocityY( 4 and -4), if changed enemies have to be changed too.
;;          Enemies must be generated in pair positions, in other case they will never reach 0 in move_to
;;      and will not be able to change to stand_by
;;
;;====================================================================================================


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Move_to
;;   The entity changes its velocity to move_to
;;   an X, Y location (e_ai_aim_x, e_ai_aim_y)
;; INPUT:
;;     IX = Pointer to the entity
;; DESTROYS: AF, IY
;;
sys_ai_move_to::
    ld      e_vx(ix), #0                    ;; Change enemy velx
    ld      e_vy(ix), #0                    ;; Change enemy vely

    ;; if ( obj_x > x ) ...
    ;; if ( obj_x - x > 0 ) ...

    ;; UPDATE X
    ld      a, e_ai_aim_x(ix)               ;; A = obj_x
    sub     e_x(ix)                         ;; A = obj_x - x
    jr      nc, _objx_greater_or_equal      ;; if ( obj_x - x > 0 ) _objx_greater_or_equal
_objx_lesser:
        ld      e_vx(ix), #-1
        jr      _endif_x
_objx_greater_or_equal:
        jr      z, _arrived_x
        ld      e_vx(ix), #1
        jr      _endif_x
_arrived_x:
        ld      e_vx(ix), #0
_endif_x:

    ;; UPDATE Y
    ld      a, e_ai_aim_y(ix)               ;; A = obj_y
    sub     e_y(ix)                         ;; A = obj_y - y
    jr      nc, _objy_greater_or_equal      ;; if ( obj_y - y > 0 ) _objy_greater_or_equal
_objy_lesser:
        ld      e_vy(ix), #-1
        jr      _endif_y
_objy_greater_or_equal:
        jr      z, _arrived_y
        ld      e_vy(ix), #1
        jr      _endif_y
_arrived_y:
        ld      e_vy(ix), #0
        ld      a, e_vx(ix)                 ;; Load velX
        or      a                           ;; Check if it's 0
        jr      nz, _endif_y                ;; if X and Y is not zero, don't change to stand_by
            ;; X and Y are both 0
_endif_y:
    ;; It doesn't change every frame
    call    ai_control_update_time
ret


;;==========================================================================================================
;;==========================================================================================================
;;==========================================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::CreateEnemy
;;  Change enemy parameters
;;
;; INPUT:
;;     IX = Entity to the array
;;     HL = Pos where the enemy will be created (in hexadecimal) --> llega al reves pq en memoria guarda hl al reves
;;      B = Life what will the enemy have
;; DESTROYS: DE
;; USE: HL, B
;;
sys_ai_create_enemy:

        ;; Component-Type
        ld e_cmp_type(ix), #e_cmp_ai_entity
        ;; Position
        ld e_x(ix), h
        ld e_y(ix), l

        ;; Sprite
        ld de, #_sp_enemigo
        ld e_sprite_ptr_l(ix), e
        ld e_sprite_ptr_h(ix), d

        ;; Lives
        ld e_vida(ix), b
        ;; AI
        ld e_ai_aim_x(ix), #0x00
        ld e_ai_aim_y(ix), #0x00
        ld e_ai_st(ix), #e_ai_st_stand_by
        ld e_ai_pre_st(ix), #e_ai_st_stand_by
        ld e_last_pos_l(ix), #0xCC
        ld e_last_pos_h(ix), #0xCC
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::ReviveEnemy
;;  Create an enemy from one already dead
;;
;; INPUT:
;;     IX = Pointer to the entity
;;     HL = Pos where the enemy will be created (in hexadecimal)
;;      B = Life what will the enemy have
;; DESTROYS: AF, DE, IX
;; USE: HL, (B)
;;
_pos: .dw #0
sys_ai_reviveEnemy::
    ld      (_pos), hl                      ;; Save HL

_ent_array_ptr_reviveEnemy = . + 2
    ld      ix, #0x0000                     ;; first position in array --> Player

    ld      de, #e_sizeof
    add     ix, de                          ;; Plant position
    add     ix, de                          ;; First bullet position
    add     ix, de
    add     ix, de
    add     ix, de
    add     ix, de
    add     ix, de                          ;; Last bullet position
    add     ix, de
_loopCreate:

    ld      a, e_w(ix)                      ;; A = Entity.witdh
    cp      #e_w_invalid                    ;; if (Entity.witdh == invalid)
    ret     z                               ;; return

    ld      a, e_cmp_type(ix)               ;; check if it is dead
    cp      #e_cmp_Dead
    jr      nz, _no_AI_dead

    ld      a, e_x(ix)                      ;; check positionX is 75(were dead enemies are located)
    cp      #75
    jr      nz, _no_AI_dead

_AI_dead:                                   ;; enemy is dead
        ;; IX = Entity_t
        ;; HL = Pos
        ;; B = Life
        ld      hl, (_pos)                  ;; H -> y and L -> x
        call    sys_ai_create_enemy
        ret                                 ;; exit, enemy reset
_no_AI_dead:
    ;; Jump to next entity
    ld      de, #e_sizeof
    add     ix, de

jr      _loopCreate

