;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------


;;
;; ENTITY COLLISION DETECTION SYSTEM
;;
.include "entity_manager.h.s"
.include "cpctelera.h.s"
.include "entity_collision_manager.h.s"
.include "physics_system.h.s"
.include "collision_system.h.s"

.module sys_collision

;;==============================================================================================
;; AI Control System Constants

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Init
;;  Inits the AI control system
;; INPUT:
;;     IX = Pointer to the entity array
;;
sys_collision_init::
      call  man_entity_collision_getArrayHL
      ld    (_ent_array_ptr), hl
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_AI_CONTROL::Update
;;   Update all AI entities
;; INPUT:
;;     IX = Pointer to the entity array
;;      A = Number of elements in the array
;; DESTROYS: AF, BC, DE, IX
;; STACK USE: 2 bytes
;;
sys_collision_update::

      _ent_array_ptr = . + 1
            ld    hl, #0x000

      ;saveSetBorder HW_GREEN

_next_ix:

      ld    e, (hl)
      inc   hl
      ld    d, (hl)
      inc   hl

      ; IF ( DE = nullptr ) EXIT
      ld    a, e
      or    d
      jr    z, _exit

      ;; Save HL
      push hl

      ld__ixl_e
      ld__ixh_d

_next_iy:

      ld    e, (hl)
      inc   hl
      ld    d, (hl)
      inc   hl

      ; IF ( DE = nullptr ) EXIT
      ld    a, e
      or    d
      jr    z, _trozo

      ld__iyl_e
      ld__iyh_d

      ;;Check collisions between entities IX and IY
      ;; DESTROYS: AF, BC

      call  physics_sys_ent_collision

      jr    _next_iy

_trozo:
      pop   hl
      jr    _next_ix

_exit:
      ;saveSetBorder HW_WHITE
      ret
