;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------

;;
;; COLLISIONABLE ENTITY MANAGER
;;
.include "entity_manager.h.s"
.include "cpctelera.h.s"

.module entity_collisionable_manager

;;==============================================================================================
;; Manager Member Variables

;; -------- Pointer arrays requiered by systems --------------
DefineComponentPointersArrayStructure_Size _collision, max_entities    ;; Poner  max_entities+1  para asegurar que tengo el maximo de entidades y un null al final

;;==============================================================================================
;;==============================================================================================
;; Manager Public Functions

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_collision::Init
;;   Initializes the Collision Entity pointer vector
;; INPUT: --
;; DESTROYS: AF, BC, DE, HL
;; RETURNS:
;;     HL: Pointer to the start of the pointer array
;;
man_entity_collision_init::
    ;; Set POinter to Array_End to collision_Ptr_Array[0]
    ld      hl, #_collision_ptr_array          ;; HL Points to collision_Ptr_Array[0]
    ld      (_collision_ptr_pend), hl          ;; collision_Ptr_pend = &collision_Ptr_Array[0]

    ;; Set all values of the pointer array to nullptr
    ;; Filling in the array with 0's
    xor      a                          ;; A = 0
    ld    (hl), a                       ;; Set first byte of the array
    ld       d, h
    ld       e, l                       ;; DE = HL+1 (Point to next byte)
    inc     de                          ;; Start copying 0's there
    ld      bc, #2*max_entities-1       ;; We need to copy 2 bytes for max_entities

    ldir                                ;; [10*(max_entities-1)] Fill in the array

    ;; BEWARE! --> DO NOT ADD CODE HERE
    ;; This function does not end here,
    ;; requires man_entity_collision_getArrayHL to be
    ;; inmediately after to perform
    ;; HL = &collision_Ptr_Array[0] and Return
    ;; not repeating code and not requiring a call or jump

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_collision::GetArrayHL
;;   Loads HL with the address where the pointer
;;  array starts and returns it
;; INPUT: --
;; DESTROYS: HL
;; RETURNS:
;;     HL: Pointer to the array of pointers (collision_Ptr_Array)
;;
man_entity_collision_getArrayHL::
    ld      hl, #_collision_ptr_array          ;; HL = &collision_Ptr_Array[0]
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAN_ENTITY_collision::Add
;;   Adds a new collision entity to the Ptr Array
;; INPUT:
;;      IX: Pointer to the new entity to be added
;; DESTROYS: A, HL
;; RETURNS:
;;     HL: Pointer to the end of collision_Ptr_Array (&collision_Ptr_Array[END])
;;
man_entity_collision_add::
    ;; Store IX in collision_Ptr_Array[PEND]
    ;; (First free element in the collision_Ptr_Array)
    ld      hl, (_collision_ptr_pend)          ;; HL = &collision_Ptr_Array[PEND]
    ld__a_ixl                           ;; Copio la parte baja de HL (L)
    ld    (hl), a
    inc     hl
    ld__a_ixh                           ;; Copio la parte alta de HL (H)
    ld    (hl), a

    ;; Move collision_Ptr_pend + 2 bytes to point to the next avcollisionlable element, which must be a nullptr
    inc     hl                          ;; Apunto al siguiente puntero del array
    ld      (_collision_ptr_pend), hl          ;; Actualizo el puntero al final
ret