;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------

;;
;;  ENTITY MANAGER
;;

;; GLOBAL FUNCTIONS
.globl entity_man_init
.globl entity_man_new
.globl entity_man_create
.globl entity_man_getArray
;; EASTEREGG
.globl entity_man_changeSprite
.globl entity_man_eastereggUpdate
.globl entity_man_erraseEastereggMessage
.globl entity_man_getEastereggA
;; BALAS
.globl entity_man_newBullet
.globl entity_man_deleteAllBullets
;; VIDAS
.globl entity_man_dec_hearts
.globl entity_man_inc_vidas
.globl entity_man_dec_cooldown
.globl entity_man_Reset_cooldown
.globl entity_man_newPower
.globl entity_man_errasePowerUP
.globl entity_man_dec_heart_IX
.globl entity_man_dec_heart_IY
;;COLLISIONS
.globl entity_man_detectType
.globl entity_man_updatePair
;;PLAYER
.globl entity_man_setPlayerPos

.globl entity_man_getDefeatedEnemies
.globl inc_defeatedEnemies
.globl entity_man_resetDefeatedEnemies

;;  GLOBAL CONSTANTS
first_heart_pos = 0xcee5
betweenHearts = 5
cooldownCollision = 15
;; MACROS
max_entities == 25
max_bullets == 5

;;=============================================================================================
;;=============================================================================================
;;=============================================================================================
;; NEW DOCUMENT ENTITY MANAGER - MACROS
;;



;;
;; ENTITY COMPONENT
;;       1B Component-Type          (cmp_type)
;;       2B Position                (x, y)
;;       2B Velocity                (vx, vy)
;;       2B Size                    (w, h)
;;       2B Direction               (x, y)
;;       2B Sprite Pointer          (sprite_ptr)
;;       2B Last Video Pointer      (last_pos)
;;       1B Vida                    (vida)
;;       1B Cooldown                (cooldown)
;;       2B AI-AIM Target           (ai_aim_x, ai_aim_y)
;;       1B AI-Status               (ai_st)
;;       1B AI-Previous-Status      (ai_pre_st)
;;       2B AI-Next-Patrol-Step     (ai_patrol_step) Pointer
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Easily assign offsets to Entity_t members without error
;;    It adds sizes of types to an offset constant that assings to
__off = 0
.macro DefOffset  _size, _name
   _name = __off
   __off = __off + _size
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Easily define an enumeration estarting in 0
;;    It adds sizes of types to an offset constant that assings to
;; - Define enumeration name
.macro DefEnum  _name
   _name'_offset = 0                            ;;'
.endm
;; - Define enumeration element for an enumeration name
.macro Enum _enumname, _element
   _enumname'_'_element = _enumname'_offset
   _enumname'_offset = _enumname'_offset + 1    ;;'
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Default constructor for Entity_t
;;
.macro DefineCmp_Entity_default
   DefineCmp_Entity 0, 0, 0, 0, e_w_invalid, 1, e_cmp_default
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Defines a Array of Entity_t with default values
;;
.macro DefineCmpArray_Entity _N
   .rept _N
      DefineCmp_Entity_default
   .endm
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Defines a New Entity_t estructure
;;     All  Entity_t data together to simplify access, at the cost of
;;
.macro DefineCmp_Entity _x, _y, _vx, _vy, _w, _h, _cmp_type, _sprite_ptr, _dir_x, _dir_y, _vida, _cooldown, _aist
   .narg __argn
   .if __argn - 13      ;; Validate parameter number
      .error 1
   .else
      ;;----- Type of component
      .db _cmp_type     ;; Types of components the entity has (bit-array)
      ;;;; CMP: Position
      .db   _x, _y      ;; Position
      ;;;; CMP: Velocity
      .db   _vx, _vy    ;; Velocity
      ;;;; CMP: Render
      .db   _w, _h      ;; Size
      .dw   _sprite_ptr ;; Sprite Pointer
      .dw   0xCCCC      ;; Last Video Memory Pointer Value
                        ;; Default to 0xCCCC value to ease memory video
      ;;;; CMP: Dir x and y
      .db _dir_x, _dir_y
      ;;;; CMP: Vida
      .db _vida
      ;;;; CMP: Cooldown
      .db _cooldown
      ;;;; CMP: AI
      .db   0x00, 0x00  ;; AI-aim (ai_aim_x, ai_aim_y)
      .db   _aist       ;; AI-status (ai_st)
      .db   _aist       ;; AI-previous status (ai_pre_st)
      ;.db   0x00
      .dw   nullptr     ;; Next AI Patrol Step (ai_patrol_step)
   .endif
.endm

;;--------------------------------------
;;-- Entity_t offsets
;;--------------------------------------
DefOffset 1, e_cmp_type       ;; Component-Type

DefOffset 1, e_x              ;; CMP: Position
DefOffset 1, e_y

DefOffset 1, e_vx             ;; CMP: Velocity
DefOffset 1, e_vy

DefOffset 1, e_w              ;; CMP: Position
DefOffset 1, e_h
DefOffset 1, e_sprite_ptr_l
DefOffset 1, e_sprite_ptr_h
DefOffset 1, e_last_pos_l
DefOffset 1, e_last_pos_h

DefOffset 1, e_dir_x          ;; CMP: Dir
DefOffset 1, e_dir_y

DefOffset 1, e_vida           ;; CMP: Vida

DefOffset 1, e_cooldown       ;; CMP: Cooldown

DefOffset 1, e_ai_aim_x       ;; CMP: AI
DefOffset 1, e_ai_aim_y
DefOffset 1, e_ai_st
DefOffset 1, e_ai_pre_st
DefOffset 1, e_ai_patrol_step_l
DefOffset 1, e_ai_patrol_step_h

DefOffset 0, e_sizeof

;;--------------------------------------
;;-- Entity_t AI-status enum
;;--------------------------------------
DefEnum e_ai_st
Enum e_ai_st noAI
Enum e_ai_st stand_by
Enum e_ai_st move_to
Enum e_ai_st patrol
Enum e_ai_st planta
Enum e_ai_st bullet
Enum e_ai_st powerupVida
;;--------------------------------------
;;-- Entity_t Component Types
;;--------------------------------------
e_cmp_Dead      = 0x00         ;; 0000000000
e_cmp_AI        = 0x01         ;; 0000000001
e_cmp_Render    = 0x02         ;; 0000000010
e_cmp_Physics   = 0x04         ;; 0000000100
e_cmp_default   = e_cmp_Render  | e_cmp_Physics    ;; Para que tenga los dos componentes, el or suma
e_cmp_ai_entity = e_cmp_default | e_cmp_AI
;;--------------------------------------
;;-- Entity_t Status enum
;;--------------------------------------
e_w_invalid = 0xFF  ;; Entity width=-1 means invalid entity
e_t_invalid = 0x00

;;--------------------------------------
;;-- Utility definitions
;;--------------------------------------
nullptr = 0x0000


;;=============================================================================================
;;=============================================================================================
;;
;;  COMPONENT ARRAY DATA STRUCTURE
;;

;;---------------------- Create Entities in Game Manager ------
.macro DefineEntity _name, _x, _y, _vx, _vy, _w, _h, _cmp_type, _sprite_ptr, _dir_x, _dir_y, _vida, _cooldown, _aist
_name::
   DefineCmp_Entity _x, _y, _vx, _vy, _w, _h, _cmp_type, _sprite_ptr, _dir_x, _dir_y, _vida, _cooldown, _aist
.endm

.macro DefineEntityArray _name,_N
_name::
   .rept _N
      DefineCmp_Entity 0xDE, 0xAD, 0xDE, 0xAD, 0xDE, 0xAD, 0xDE, 0xAD, 0xDE, 0xAD, 0xDE, 0xAD, e_ai_st_noAI
   .endm
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;----------------------FOR AI------
;; INPUTS:
;;    _Tname: Name of the component type
;;        _N: Maximum number of pointers to store
;;
.macro DefineComponentPointersArrayStructure_Size _Tname, _N
   _Tname'_ptr_pend:    .dw   _Tname'_ptr_array       ;;' Pointer to the entity
   _Tname'_ptr_array::                                ;;' Array of pointers
   .rept _N
      .dw   0x0000                                    ;; Relleno con nullptr
   .endm
.endm
