;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------

;;
;;  ENTITY MANAGER
;;
.module entityManager
.include "entity_manager.h.s"
.include "cpctelera.h.s"
.include "render_system.h.s"
.include "game_manager.h.s"
.include "cpctelera_functions.h.s"
.include "entity_collision_manager.h.s"

.globl _sp_gatoComido
.globl _sp_gatete
.globl _sp_fondoAzul
.globl _sp_balaBlue
.globl _sp_hair
.globl _sp_heartGreen
.globl _sp_heartBlue
frase_planta1: .asciz "Down you go!"
frase_planta2: .asciz "mmmmmmmhhhh!!"

_num_entities::  .db 0
_last_entity_ptr::  .dw _entity_array
_easterEgg:: .db 0
_timer: .db 0

defeatedEnemies:: .db #0

DefineEntityArray  _entity_array, max_entities
;;  --- Entity Manager newBullet ---
;;  looks for the first dead bullets and resets it
;;  INPUT:  HL -> x, vx
;;          DE -> y, vy
;;  DESTROYS:
;;          DE, A, IY
;;  RETURNS:
;;          A = -1 no dead bullets
;;          A =  0 bullet reseted
entity_man_newBullet::
    push de
    ld iy, #_entity_array
    ld de,#e_sizeof
    add iy, de
    add iy, de            ;;bullets start at pos 3 of entity array
    ld a, #max_bullets
    pop de
_update_loop:
    ld b, a
    ld a, e_cmp_type(iy)
    cp #e_cmp_Dead
    jr nz, nextBulletArray
    ;if it's 0 then is a dead bullet
        ;;change el cmp
        ld e_cmp_type(iy), #e_cmp_default
        ;;change vel y pos
        ld e_x(iy),  h
        ld e_vx(iy), l
        ld e_y(iy),  d
        ld e_vy(iy), e
        ;;change sprite
        ld de, #_sp_hair

        ld  e_sprite_ptr_l(iy), e
        ld  e_sprite_ptr_h(iy), d
        ret
    ;;next bullet
    nextBulletArray:
    ld a, b
    dec a
    cp #0
    jr z, endloopBllt
    push de
    ld de,#e_sizeof        ;; DE = sizeof(Entity_t)
    add iy, de             ;; IY Points to next Entity_t
    pop de
    jp _update_loop            ;; Continue updating
endloopBllt:
ld a, #-1
ret
;;  --- Entity Man New Powerup --
;; resets powerup
;;INPUT: HL-> new position
;;  DESTROYS:
;;          DE, A, IY
;;  RETURNS:
;;          A = -1 no powerup
;;          A =  0 powerup reseted
entity_man_newPower::
    ld iy, #_entity_array
    ld de,#e_sizeof        ;; DE = sizeof(Entity_t)
    add iy, de             ;; player
    add iy, de             ;; plant
    add iy, de             ;; bullets
    add iy, de
    add iy, de
    add iy, de
    add iy, de

    ld a, e_cmp_type(iy)
    cp #e_cmp_Dead
    jr nz, finH
        ;;change cmp
        ld e_cmp_type(iy), #e_cmp_default
        ;;change vel and pos
        ld e_x(iy),  h
        ld e_y(iy),  l

        ld e_last_pos_l(iy), #0xcc
        ld e_last_pos_h(iy), #0xcc
        ;change sprite
        ld de, #_sp_heartGreen

        ld  e_sprite_ptr_l(iy), e
        ld  e_sprite_ptr_h(iy), d
        ret
finH:
ld a, #-1
ret
;;  --- ENTITY MANAGER INITIALICER  ---
;;  Allows restarting the game
;;
;;  INPUT:  NONE
;;  DESTROYS:   AF, HL
;;  RETURNS:
;;          HL: Pointer to the start of the array
entity_man_init::
    ;; Reset all component vector values
    call    man_entity_collision_init

    xor     a, a                    ;;reset A
    ld      (_num_entities), a       ;;set number of entities to 0
    ld      (_easterEgg), a
    ld      (_timer), a

    ld      hl, #_entity_array      ;;
    ld      (_last_entity_ptr), hl  ;;set the pointer to the fist position

    ;; First entity must be marked as no entity as it is the last valid entity in the array
    ex      de, hl                  ;; Saved HL into DE
    ld      hl, #e_w                ;; HL = Offset of the e_w in Entity.witdh
    add     hl, de                  ;; HL  =  Entity +  e_w (points to array)
    ld      (hl), #e_w_invalid      ;; Entity[0].e_w = e_width noEntity
    ex      de, hl                  ;; Restore HL = &Entity[0]

ret

;;  --- ENITITY MANAGER NEW ---
;;  Creates new entity without values and updates _num_entities and _last_entity_ptr
;;  INPUT: NONE
;;  DESTROYS:   HL, DE, BC, F
;;  RETURNS:
;;          DE -> last entity pointer
;;          BC -> enitity size
entity_man_new::
    ld hl,#_num_entities
    inc (hl)                    ;;Increases num_enitities value

    ld hl, (_last_entity_ptr)   ;;HL = last entity mem. position
    ld d, h                     ;;DE register points to new entitys beginning
    ld e, l                     ;;We cannot exchange DE and HL values because HL is needed later

    ld bc, #e_sizeof            ;;BC = enitity size
    add hl, bc                  ;;
    ld (_last_entity_ptr), hl   ;;Updates _last_entity_ptr

ret

;;  --- ENTITY MANAGER CREATE
;;  INPUT: NONE
;;  DESTROYS:   HL, DE, BC, AF
;;  RETURNS:
;;          HL -> last entity pointer
;;          BC -> enitity size
entity_man_create::
    push    hl                  ;;Saves HL
    call    entity_man_new

    ld__ixh_d                   ;;loads D in IX FIRTST 8 bytes   [Non documented]
    ld__ixl_e                   ;;loads E in IX LASTS 8 bytes    [Non documented]

    pop     hl
    push    de
    ldir

    ld      a, e_cmp_type(ix)
    and     #e_cmp_Physics
    call    nz, man_entity_collision_add

    ld      hl, #e_w

    add     hl, de
    ld      (hl), #e_w_invalid
    pop     hl

ret

;;  --- Errase Heart  ---
;;  Draws a box on top of the last lost heart
;;  INPUT::
;;          A -> number of hearts
;;
entity_man_erraseheart::

    ld a, e_vida(ix)
    ld hl, #first_heart_pos
    ld de, #betweenHearts
loopVida:
    cp #0
    jr z, endlooopVida
    add hl, de
    dec a
    jr loopVida
endlooopVida:
    ex de, hl
    xor a
    ld bc, #0x0c04 ;hw
    call cpct_drawSolidBox_asm
ret

;;  --- Change Sprite ---
;;  Its only purpose for the easteregg
;;  Changes player Sprite
entity_man_changeSprite::
    ld ix, #_entity_array
    ld de, #_sp_gatete
    ld a,  e_sprite_ptr_l(ix)
    cp a, e
    jr nz, notSame                  ;;if it's not the same sprite, then i make it the same
    ld a,  e_sprite_ptr_h(ix)       ;;if it's the same compare high
    cp a, d
    jr nz, notSame
Same:                               ;;if it's the same change sprite to gatete_comido
    ld de, #_sp_gatoComido

    ld  e_sprite_ptr_l(ix), e
    ld  e_sprite_ptr_h(ix), d

    jr endSame
notSame:
    ld de, #_sp_gatete

    ld  e_sprite_ptr_l(ix), e
    ld  e_sprite_ptr_h(ix), d
endSame:
ret
;;  --- Detect Type --
;;  Detects type of entity, B needs to be 0
;;  INPUT:
;;          A -> e_ai_st
;;  OUTPUT:
;;          B -> 1 Player, 2 Plant, 4 Enemy, 8 Bullet, 14 powerHeart
entity_man_detectType::

    cp #e_ai_st_noAI
    jr nz, noPlayer
Player:
    set 0, b
    ret
noPlayer:

    cp #e_ai_st_planta
    jr nz, noPlanta
Planta:
    set 1, b
    ret
noPlanta:

    cp #e_ai_st_bullet
    jr nz, noBullet
Bullet:
    set 3, b
    ret
noBullet:

    cp #e_ai_st_powerupVida
    jr nz, noPower
Power:
    set 1, b
    set 2, b
    set 3, b
    ret
noPower:

    cp #e_ai_st_stand_by
    jr z, EnemyEnt
    cp #e_ai_st_move_to
    jr z, EnemyEnt
    cp #e_ai_st_patrol
    jr nz, noEnemyEnt
EnemyEnt:
    set 2, b
noEnemyEnt:
endDetect:
ret
;;  --- Update  ---
;;  INPUT:
;;      B -> 0011 Plant-Player, 1100 Bullet-Enemy, 0101 Enemy-Player
;;      player-planta -> 0011 = 3
;;      enemy-bullet  -> 1100 = C
;;      player-enemy  -> 0101 = 5
;;      player-heart  -> 1111 = F
entity_man_updatePair::
    ld a, b
    cp #3            ;;player-plant
    jr nz, #noEasteregg
Easteregg:
    ld a, (_easterEgg)
    cp # 0
    jr nz,noEasteregg
    ld a, e_cooldown(ix)
    cp #0
    jr nz, noEasteregg
        call entity_man_dec_hearts
        ld   a, e_vida(ix)
        cp   a, #0
        jr   z, noEasteregg
        call entity_man_Reset_cooldown
        call entity_man_changeSprite
        ld e_vx(ix), #0
        ld e_vy(ix), #0
        push ix
            ld  hl, #0xc6f6
            ld  iy, #frase_planta1
            call cpct_drawStringM0_asm
            ld  hl, #0xc746
            ld  iy, #frase_planta2
            call cpct_drawStringM0_asm
        pop ix
    ld a, #1
    ld   (_easterEgg), a
    ld a, #100
    ld   (_timer), a

    ret
noEasteregg:
    cp #5            ;;player-enemy
    jr nz, noEnemy
        ld a, e_cooldown(ix)

        cp #0
        jr nz, noEnemy_bullet
            call    entity_man_dec_hearts
            call    entity_man_Reset_cooldown
            jr      noEnemy_bullet
noEnemy:
    cp  #0x000c       ;;enemy-bullet
    jr nz, noEnemy_bullet
        ;;Enemy and bullet collide
        call entity_man_dec_heart_IX
        call entity_man_dec_heart_IY
noEnemy_bullet:

    cp #0xF            ;;player-heart
    jr nz, noIncVida
        call entity_man_inc_vidas
        call entity_man_errasePowerUP
noIncVida:

ret

;;      --- DEC VIDA ---
;;  Decreases hearts of an entity
;;  INPUT::
;;          IX -> pointer to entity
;;  OUTPUT::
;;          A  -> number of lives
entity_man_dec_heart_IX::
    ld a, e_vida(ix)
    cp #1

    jr nz, morethanoneLife
        ;delete entity          --> Enemy
        ld  e_cmp_type(ix), #e_cmp_Dead     ;;change state to dead
        ld  e_x(ix), #65                    ;;change posx
        ld  e_y(ix), #186                   ;;change posy
        ;;Change el sprite
        ld  de, #_sp_balaBlue
        ld  e_sprite_ptr_l(ix), e
        ld  e_sprite_ptr_h(ix), d

        jr  end
morethanoneLife:
    dec e_vida(ix)
end:
ret

entity_man_dec_heart_IY::
    ld a, e_vida(iy)
    cp #1

    jr nz, _morethanoneLife
        ;delete entity           --> LIFE
        ;;incremento el contador de enemigo derrotado
        call inc_defeatedEnemies

        ld  e_cmp_type(iy), #e_cmp_Dead
        ld  e_x(iy), #75
        ld  e_y(iy), #186
        ;;change sprite
        ld  de, #_sp_fondoAzul
        ld  e_sprite_ptr_l(iy), e
        ld  e_sprite_ptr_h(iy), d
        jr  _end
_morethanoneLife:
    dec e_vida(iy)
_end:
ret

;;  --- Entity man ---
;;  Sets all bullets to dead
;;
entity_man_deleteAllBullets::
    push de
    ld ix, #_entity_array
    ld de,#e_sizeof        ;; DE = sizeof(Entity_t)
    add ix, de             ;; IX Points to next Entity_t
    add ix, de             ;; IX Points to next Entity_t  / first bullet in pos 3
    ld a, #max_bullets
    pop de
_update_loopb:
    ld b, a
    ld a, e_cmp_type(ix)
    cp #e_cmp_default
    jr nz, nxtbullet
        ;;if it's alive i kill it
        call entity_man_dec_heart_IX

    nxtbullet:
    ld a, b
    dec a
    cp #0
    jr z, endb  ;if 0, out of the loop
    push de
    ld de,#e_sizeof             ;; DE = sizeof(Entity_t)
    add ix, de                  ;; IX Points to next Entity_t
    pop de
    jp _update_loopb            ;; Continue updating
endb:
ret

entity_man_eastereggUpdate::
    ld a, (_timer)
    cp a, #0
    jr z, noDecTimer
        dec a
        ld (_timer), a
noDecTimer:
ret

entity_man_erraseEastereggMessage::
    ld de, #0xc6f6  ;pos
    xor a
    ld bc, #0x0f33  ;hw
    call cpct_drawSolidBox_asm
ret

;;  --- Entity Man Errase PowerUP ---
;;  Sets PowetUP to Dead
;;  DESTROYS:
;;          IY, DE
entity_man_errasePowerUP::
    ld iy, #_entity_array
    ld de,#e_sizeof        ;; DE = sizeof(Entity_t)
    add iy, de             ;; it is in position 8
    add iy, de
    add iy, de
    add iy, de
    add iy, de
    add iy, de
    add iy, de
    ld  e_cmp_type(iy), #e_cmp_Dead
    ld e_x(iy), #75
    ld e_y(iy), #188
    ld  de, #_sp_heartBlue
    ld  e_sprite_ptr_l(iy), e
    ld  e_sprite_ptr_h(iy), d
ret

entity_man_setPlayerPos::
    ld ix, #_entity_array
    ld e_x(ix), a
    ld e_y(ix), b
ret

;;  --- Dec Cooldown ---
;;  Decrements cooldown  util it's 0
;;
entity_man_dec_cooldown::
    ld a, e_cooldown(ix)

    cp #0
    jr z, noDec
Decrement:
    dec  e_cooldown(ix)
noDec:
ret

;;  --- Reset Cooldown ---
;;  Resets Cooldown value
;;
entity_man_Reset_cooldown::
    ld e_cooldown(ix), #cooldownCollision
ret

;;      --- DEC VIDA ---
;;  Decreases hearts of an entity
;;  INPUT::
;;          IX -> pointer to entity
;;  OUTPUT::
;;          A  -> number of lives
entity_man_dec_hearts::
    ld a, e_vida(ix)
    cp #1
    jr nz, canDecrease
        ld  a, #3
        call game_man_setState
        ;cpctm_clearScreen_asm #0x00
        call cpct_waitVSYNC_asm
        call render_sys_gameover
    jr endDec
canDecrease:
    dec e_vida(ix)
    call entity_man_erraseheart
endDec:
ret
;;      --- INC VIDA ---
;;  Increases hearts of an entity
;;  INPUT::
;;          IX -> pointer to entity
;;  OUTPUT::
;;          A  -> number of lives
entity_man_inc_vidas::
    ld a, e_vida(ix)
    cp #3
    jr nz, canIncrease
    jr endInc
canIncrease:
    inc e_vida(ix)
endInc:
ret
;;Getters
;;  Returns:
;;          IX -> Pointer to entity array
;;           A -> Total entities
entity_man_getArray::
    ld   ix, #_entity_array            ;; Cambiar por:   ld   hl, #_entity_array
    ld    a, (_num_entities)           ;; Esto borrar
ret

entity_man_getEastereggA::
    ld a, (_timer)
ret

entity_man_getDefeatedEnemies::
    ld  a, (defeatedEnemies)
ret

inc_defeatedEnemies::
    ld  a, (defeatedEnemies)
    inc a
    ld  (defeatedEnemies), a
ret

entity_man_resetDefeatedEnemies::
    ld  a, #0
    ld  (defeatedEnemies), a
ret