;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------

;; includes

.module gameManager
.include "entity_manager.h.s"
.include "render_system.h.s"
.include "input_system.h.s"
.include "physics_system.h.s"
.include "game_manager.h.s"
.include "ai_control_system.h.s"
.include "cpctelera_functions.h.s"
.include "cpctelera.h.s"
.include "collision_system.h.s"
.include "assets/pack_1.h.s"

.globl _sp_gatete
.globl _sp_fondoAzul
.globl _sp_balaBlue
.globl _sp_enemigo
.globl _sp_planta
.globl _sp_hair
.globl _sp_heartGreen

.globl _nivel01_W

state::         .db #2
boolean_tiles:: .db #1
nivel::         .db #1
compression::   .db #1

timeStartCooldownlv1 = 100
timeStartCooldown = 60
spawnCooldown:: .db #timeStartCooldownlv1
enemyNumber::   .db #0



ent1: DefineEntity player,  8, 16, 1,  1, 4,  8,   e_cmp_default,     _sp_gatete, 1, 0, 3, cooldownCollision,     e_ai_st_noAI
ent2: DefineEntity planta, 76, 72, 0,  0, 4, 12,   e_cmp_default,     _sp_planta, 1, 0, 1,                 0,   e_ai_st_planta
ent3: DefineEntity enemy1, 76, 80, 1,  0, 4,  8, e_cmp_ai_entity,    _sp_enemigo, 0, 0, 1,                 0, e_ai_st_stand_by

ent4: DefineEntity bullet, 50, 10, 0,  0, 3, 6,    e_cmp_default,       _sp_hair, 0, 0, 1,                 0,    e_ai_st_bullet

;END
ent5: DefineEntity enemy5, 76, 80, 0,  0, 4,  8, e_cmp_ai_entity,    _sp_enemigo, 0, 0, 1,                 0, e_ai_st_stand_by
ent6: DefineEntity enemy6, 76, 80, 0,  0, 4,  8, e_cmp_ai_entity,    _sp_enemigo, 0, 0, 1,                 0, e_ai_st_stand_by
ent7: DefineEntity enemy7, 76, 80, 0,  0, 4,  8, e_cmp_ai_entity,    _sp_enemigo, 0, 0, 1,                 0, e_ai_st_stand_by
ent8: DefineEntity enemy8, 76, 124, 0,  0, 4,  8, e_cmp_ai_entity,    _sp_enemigo, 0, 0, 1,                 0, e_ai_st_stand_by
ent9: DefineEntity enemy9, 76, 44, 0,  0, 4,  8, e_cmp_ai_entity,    _sp_enemigo, 0, 0, 1,                 0, e_ai_st_stand_by

;DEBUG
ent10: DefineEntity corazon, 50, 50, 0,  0, 4, 12,   e_cmp_default,     _sp_heartGreen, 1, 0, 1,                 0,   e_ai_st_powerupVida
;ENDDEBUG

decompress_buffer     = 0x3AAA
levelmaxsize          = 0x1268
decompress_buffer_end = decompress_buffer + levelmaxsize -1
tileset_ptr           = decompress_buffer + 4200
tilemap_ptr           = decompress_buffer + 0

game_man_init::

    ;;init managers
    ld      a, #1
    ld      (nivel), a

    ld      a, #1
    call    game_man_setBooleanTiles

    ld      a, #0
    ld  (enemyNumber), a

;   call patrol_man_init
    call    entity_man_resetDefeatedEnemies
    call    entity_man_init        ;; HL = EntityArray


    ;;;init systems
    call    ai_control_sys_init
    call    physics_sys_init
    call    input_sys_init
    call    render_sys_init        ;; Modifies HL      ;;bellow input_init
    call    sys_collision_init

    ;;init entities
    ld      hl, #ent1
    call    entity_man_create
    ;;plant/easteregg
    ld      hl, #ent2
    call    entity_man_create
    ;;bullets
    ;; the number of bullet needs to be max_bullets (currently = 5)
    ld      hl, #ent4
    call    entity_man_create
    ld      hl, #ent4
    call    entity_man_create
    ld      hl, #ent4
    call    entity_man_create
    ld      hl, #ent4
    call    entity_man_create
    ld      hl, #ent4
    call    entity_man_create

    ;;powerup Heart
    ld      hl, #ent10
    call    entity_man_create
ret


game_man_createEnemies::

    ld      a, (nivel)
    ;nivel1
    cp      #1
    jr      nz, nivelDos
        ;CREAR ENEMIGOS NIVEL 1
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter:
            ;; Primero reseteo el cooldown
            ld      a, #timeStartCooldownlv1
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo (3 para el primer nivel)
            ld      a, (enemyNumber)
            cp      a, #3
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo
                ld      hl, #ent3
                call    entity_man_create
                jp      _fin
            _enemyTwo:
                cp      a, #2
                jr      nz, _enemyThree
                ld      hl, #ent5
                call    entity_man_create
                jp      _fin
            _enemyThree:
                ld      hl, #ent6
                call    entity_man_create
        jp      _fin

    nivelDos:
    cp      #2
    jr      nz, nivelTres
        ;CREAR ENEMIGOS NIVEL 2
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l2
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l2:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #3
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l2
                ld      hl, #ent7
                call    entity_man_create
                jp      _fin
            _enemyTwo_l2:
                cp      a, #2
                jr      nz, _enemyThree_l2
                ld      hl, #ent8
                call    entity_man_create
                jp      _fin
            _enemyThree_l2:
                ld      hl, #ent9
                call    entity_man_create
        jp  _fin


    nivelTres:
    cp      #3
    jp      nz, nivelCuatro
        ;CREAR ENEMIGOS NIVEL 3
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l3
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l3:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #4
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l3
                ;; This macro reutilize a dead enemy.
                ;; The first value is the position (hex) where the enemy will be setted
                ;; and the second is the life he will have
                reviveEnemy #0X4C50, #1
                jp      _fin
            _enemyTwo_l3:
                cp      a, #2
                jr      nz, _enemyThree_l3
                reviveEnemy #0X0450, #1
                jp      _fin
            _enemyThree_l3:
                cp      a, #3
                jr      nz, _enemyFour_l3
                reviveEnemy #0X4C7C, #1
                jp      _fin
            _enemyFour_l3:
                reviveEnemy #0X042C, #1
        jp  _fin


    nivelCuatro:
    cp      #4
    jp      nz, nivelCinco
        ;CREAR ENEMIGOS NIVEL 4
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l4
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l4:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #4
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l4
                reviveEnemy #0X2802, #1
                jp      _fin
            _enemyTwo_l4:
                cp      a, #2
                jr      nz, _enemyThree_l4
                reviveEnemy #0X28A0, #1
                jp      _fin
            _enemyThree_l4:
                cp      a, #3
                jr      nz, _enemyFour_l4
                reviveEnemy #0X047C, #1
                jp      _fin
            _enemyFour_l4:
                reviveEnemy #0X4C2C, #1
        jp  _fin


    nivelCinco:
    cp      #5
    jp      nz, nivelSeis
        ;CREAR ENEMIGOS NIVEL 5
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l5
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l5:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #5
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l5
                reviveEnemy #0X042C, #1
                jp      _fin
            _enemyTwo_l5:
                cp      a, #2
                jr      nz, _enemyThree_l5
                reviveEnemy #0X047C, #1
                jp      _fin
            _enemyThree_l5:
                cp      a, #3
                jr      nz, _enemyFour_l5
                reviveEnemy #0X4C7C, #1
                jp      _fin
            _enemyFour_l5:
                cp      a, #4
                jr      nz, _enemyFive_l5
                reviveEnemy #0X4C2C, #1
                jp      _fin
            _enemyFive_l5:
                reviveEnemy #0X2802, #1

        jp  _fin


    nivelSeis:
    cp      #6
    jp      nz, nivelSiete
        ;CREAR ENEMIGOS NIVEL 6
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l6
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l6:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #5
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l6
                reviveEnemy #0X4C2C, #1
                jp      _fin
            _enemyTwo_l6:
                cp      a, #2
                jr      nz, _enemyThree_l6
                reviveEnemy #0X4C50, #1
                jp      _fin
            _enemyThree_l6:
                cp      a, #3
                jr      nz, _enemyFour_l6
                reviveEnemy #0X042C, #1
                jp      _fin
            _enemyFour_l6:
                cp      a, #4
                jr      nz, _enemyFive_l6
                reviveEnemy #0X0450, #1
                jp      _fin
            _enemyFive_l6:
                reviveEnemy #0X4C7C, #1
        jp  _fin

    nivelSiete:
    cp      #7
    jp      nz, nivelOcho
        ;CREAR ENEMIGOS NIVEL 7
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l7
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l7:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #6
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l7
                reviveEnemy #0X2802, #1
                jp      _fin
            _enemyTwo_l7:
                cp      a, #2
                jr      nz, _enemyThree_l7
                reviveEnemy #0X28A0, #1
                jp      _fin
            _enemyThree_l7:
                cp      a, #3
                jr      nz, _enemyFour_l7
                reviveEnemy #0X4C50, #1
                jp      _fin
            _enemyFour_l7:
                cp      a, #4
                jr      nz, _enemyFive_l7
                reviveEnemy #0X0450, #1
                jp      _fin
            _enemyFive_l7:
                cp      a, #5
                jr      nz, _enemySix_l7
                reviveEnemy #0X4C50, #1
                jp      _fin
            _enemySix_l7:
                reviveEnemy #0X0450, #1
        jp  _fin


    nivelOcho:
    cp      #8
    jp      nz, nivelNueve
        ;CREAR ENEMIGOS NIVEL 8
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l8
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l8:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #6
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l8
                reviveEnemy #0X4C2C, #1
                jp      _fin
            _enemyTwo_l8:
                cp      a, #2
                jr      nz, _enemyThree_l8
                reviveEnemy #0X28A0, #1
                jp      _fin
            _enemyThree_l8:
                cp      a, #3
                jr      nz, _enemyFour_l8
                reviveEnemy #0X0450, #1
                jp      _fin
            _enemyFour_l8:
                cp      a, #4
                jr      nz, _enemyFive_l8
                reviveEnemy #0X042C, #1
                jp      _fin
            _enemyFive_l8:
                cp      a, #5
                jr      nz, _enemySix_l8
                reviveEnemy #0X2802, #1
                jp      _fin
            _enemySix_l8:
                reviveEnemy #0X0450, #1
        jp  _fin


    nivelNueve:
    cp      #9
    jp     nz, nivelDiez
        ;CREAR ENEMIGOS NIVEL 9
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l9
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l9:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #6
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l9
                reviveEnemy #0X0450, #1
                jp      _fin
            _enemyTwo_l9:
                cp      a, #2
                jr      nz, _enemyThree_l9
                reviveEnemy #0X042C, #1
                jp      _fin
            _enemyThree_l9:
                cp      a, #3
                jr      nz, _enemyFour_l9
                reviveEnemy #0X0450, #1
                jp      _fin
            _enemyFour_l9:
                cp      a, #4
                jr      nz, _enemyFive_l9
                reviveEnemy #0X28A0, #1
                jp      _fin
            _enemyFive_l9:
                cp      a, #5
                jr      nz, _enemySix_l9
                reviveEnemy #0X2802, #1
                jp      _fin
            _enemySix_l9:
                reviveEnemy #0X4C2C, #1
        jp  _fin


    nivelDiez:
    ;CREAR ENEMIGOS NIVEL 10
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Uso  una variable de cooldown y otra de contador   ;;
        ;; Cuando el cooldown llegue a 0 aumento el contador  ;;
        ;; En funcion del contador creo un enemigo u otro     ;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ld      a, (spawnCooldown)
        cp      a, #0
        jr      z, _incCounter_l10
            dec     a
            ld      (spawnCooldown), a
            jp      _fin

        _incCounter_l10:
            ;; Reset spawnCooldown
            ld      a, #timeStartCooldown
            ld      (spawnCooldown), a
            ; El cooldown es 0, asi que comprobamos si el contador de enemigos ha llegado a su valor maximo
            ld      a, (enemyNumber)
            cp      a, #6
            jp      z, _fin
            ;; Si no ha llegado a su maximo lo aumento (por defecto esta a 0, la primera vez que se aumente será para crear el primer enemigo)
            inc     a
            ld      (enemyNumber), a
            ;; Compruebo qué enemigo hay que crear
                cp      a, #1
                jr      nz, _enemyTwo_l10
                reviveEnemy #0X0450, #1
                jp      _fin
            _enemyTwo_l10:
                cp      a, #2
                jr      nz, _enemyThree_l10
                reviveEnemy #0X042C, #1
                jp      _fin
            _enemyThree_l10:
                cp      a, #3
                jr      nz, _enemyFour_l10
                reviveEnemy #0X0450, #1
                jp      _fin
            _enemyFour_l10:
                cp      a, #4
                jr      nz, _enemyFive_l10
                reviveEnemy #0X28A0, #1
                jp      _fin
            _enemyFive_l10:
                cp      a, #5
                jr      nz, _enemySix_l10
                reviveEnemy #0X2802, #1
                jp      _fin
            _enemySix_l10:
                reviveEnemy #0X4C2C, #1

_fin:
ret


game_man_drawTiles::

    ld      a, #2
    call    game_man_setBooleanTiles

    ld      a, (compression)
    dec     a
    jr      nz, _decompressed
        ld      (compression), a
        ld      hl, #_pack_1_end
        ld      de, #decompress_buffer_end
        call    cpct_zx7b_decrunch_s_asm

_decompressed:
    ld      bc, #0x1514
    ld      de, #_nivel01_W
    ld      hl, #tileset_ptr
    call    cpct_etm_setDrawTilemap4x8_ag_asm

    ld      hl, #0xC000

    ld      a, (nivel)
    cp      #1
        ;nivel1:
        jr      nz, nivel2
        ld      de, #tilemap_ptr ;+20 nivel 2 | +40 nivel 3 | +60 nivel 4 | ... | +180 nivel 10
        jr      draw

        nivel2:
        cp      #2
        jr      nz, nivel3
        ld      de, #tilemap_ptr + 20
        jr      draw

        nivel3:
        cp      #3
        jr      nz, nivel4
        ld      de, #tilemap_ptr + 40
        jr      draw

        nivel4:
        cp      #4
        jr      nz, nivel5
        ld      de, #tilemap_ptr + 60
        jr      draw

        nivel5:
        cp      #5
        jr      nz, nivel6
        ld      de, #tilemap_ptr + 80
        jr      draw

        nivel6:
        cp      #6
        jr      nz, nivel7
        ld      de, #tilemap_ptr + 100
        jr      draw

        nivel7:
        cp      #7
        jr      nz, nivel8
        ld      de, #tilemap_ptr + 120
        jr      draw

        nivel8:
        cp      #8
        jr      nz, nivel9
        ld      de, #tilemap_ptr + 140
        jr      draw

        nivel9:
        cp      #9
        jr      nz, nivel10
        ld      de, #tilemap_ptr + 160
        jr      draw

        nivel10:
        ld      de, #tilemap_ptr + 180
        jr      draw

draw:
    call    cpct_etm_drawTilemap4x8_ag_asm
ret

game_man_update::

    call entity_man_getEastereggA
    cp #1
    jr nz, notChange
    Change:
        call entity_man_changeSprite
        call entity_man_erraseEastereggMessage
    notChange:
    cp #0
    jr nz, noInput
        call input_sys_update
        jr endnoInput
    noInput:
        call entity_man_eastereggUpdate
    endnoInput:

    call entity_man_dec_cooldown
    call ai_control_sys_update

    call game_man_createEnemies
    call sys_collision_update
    call physics_sys_update

ret

game_man_changeLevel::
    call entity_man_getArray
ret

game_man_render::
    ;just render if the state of the game is "playing"
    call  game_man_getState
    cp #1
    jr nz, noGameRender
        call render_sys_update
        call render_system_drawVida
    noGameRender:
ret

game_man_resetEnemySpawn::

    ;Reseteo las variables de aparicion de enemigos
    ld      a, #timeStartCooldown
    ld      (spawnCooldown), a

    ld      a, #0
    ld      (enemyNumber), a
ret

game_man_changeNivelHorizontal::

    ;Obtengo el nivel al que tengo que cambiar
    call    game_man_getNivel

    cp #1   ;;Si estoy en el nivel 1 paso al 2
    jr nz, nivelDosH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #3
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call    game_man_resetEnemySpawn
            call    entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #2
            call    game_man_setNivel
            ld      a, #2   ;;AJUSTAR
            ld      b, e_y(ix) ;;AJUSTAR
            call    entity_man_setPlayerPos
        jp  endH

    nivelDosH:
    cp #2   ;;Si estoy en el nivel 2 paso al 3
    jr nz, nivelTresH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #3
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #3
            call    game_man_setNivel
            call    puertaCorrectaHorizontal
            call    entity_man_setPlayerPos
        jp      endH

    nivelTresH:
    cp #3   ;;Si estoy en el nivel 3 paso al 4
    jr nz, nivelCuatroH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #4
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call    game_man_resetEnemySpawn
            call    entity_man_resetDefeatedEnemies
            ;Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #4
            call    game_man_setNivel
            call    puertaCorrectaHorizontal
            call    entity_man_setPlayerPos
        jp  endH


    nivelCuatroH:
    cp  #4  ;;Si estoy en el nivel 4 paso al 5
    jr  nz, nivelCincoH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #4
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #5
            call    game_man_setNivel
            call    puertaCorrectaHorizontal
            call    entity_man_setPlayerPos
        jp  endH

    nivelCincoH:
    cp  #5  ;;Si estoy en el nivel 5 paso al 6
    jr  nz, nivelSeisH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #5
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #6
            call    game_man_setNivel
            call    puertaCorrectaHorizontal
            call    entity_man_setPlayerPos
        jp  endH

    nivelSeisH:
    cp  #6  ;;Si estoy en el nivel 6 paso al 7
    jr  nz, nivelSiesteH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #5
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #7
            call    game_man_setNivel
            call    puertaCorrectaHorizontal
            call    entity_man_setPlayerPos
        jp  endH

    nivelSiesteH:
    cp  #7  ;;Si estoy en el nivel 7 paso al 8
    jr  nz, nivelOchoH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #6
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #8
            call    game_man_setNivel
            call    puertaCorrectaHorizontal
            call    entity_man_setPlayerPos
        jp  endH

    nivelOchoH:
    cp  #8 ;;Si estoy en el nivel 8 paso al 9
    jr  nz, nivelNueveH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #6
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #9
            call    game_man_setNivel
            call    puertaCorrectaHorizontal
            call    entity_man_setPlayerPos
        jp  endH

    nivelNueveH:
    cp  #9 ;;Si estoy en el nivel 9 paso al 10
    jr  nz, ultimoNivelH
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #6
        jp      nz, endH
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #10
            call    game_man_setNivel
            call    puertaCorrectaHorizontal
            call    entity_man_setPlayerPos
        jp  endH

    ultimoNivelH:
        ld  a, #6
        call game_man_setState
        call render_sys_win
endH:
ret

;;this can be changed without that many repetition
;;TODO change if there's enought time
game_man_changeNivelVertical::

    ;Obtengo el nivel al que tengo que cambiar
    call    game_man_getNivel

    cp #1   ;;Si estoy en el nivel 1 paso al 2
    jr nz, nivelDosV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #3
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call    game_man_resetEnemySpawn
            call    entity_man_resetDefeatedEnemies
            ; Repaintig tiles activated
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #2
            call    game_man_setNivel
            ld      a, #2   ;;AJUSTAR
            ld      b, e_y(ix) ;;AJUSTAR
            call    entity_man_setPlayerPos
        jp  endV

    nivelDosV:
    cp #2   ;;Si estoy en el nivel 2 paso al 3
    jr nz, nivelTresV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #3
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #3
            call    game_man_setNivel
            call    puertaCorrectaVertical
            call    entity_man_setPlayerPos
        jp      endV

    nivelTresV:
    cp #3   ;;Si estoy en el nivel 3 paso al 4
    jr nz, nivelCuatroV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #4
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call    game_man_resetEnemySpawn
            call    entity_man_resetDefeatedEnemies
            ;Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #4
            call    game_man_setNivel
            call    puertaCorrectaVertical
            call    entity_man_setPlayerPos
        jp  endV


    nivelCuatroV:
    cp  #4  ;;Si estoy en el nivel 4 paso al 5
    jr  nz, nivelCincoV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #4
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #5
            call    game_man_setNivel
            call    puertaCorrectaVertical
            call    entity_man_setPlayerPos
        jp  endV

    nivelCincoV:
    cp  #5  ;;Si estoy en el nivel 5 paso al 6
    jr  nz, nivelSeisV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #5
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #6
            call    game_man_setNivel
            call    puertaCorrectaVertical
            call    entity_man_setPlayerPos
        jp  endV

    nivelSeisV:
    cp  #6  ;;Si estoy en el nivel 6 paso al 7
    jr  nz, nivelSiesteV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #5
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #7
            call    game_man_setNivel
            call    puertaCorrectaVertical
            call    entity_man_setPlayerPos
        jp  endV

    nivelSiesteV:
    cp  #7  ;;Si estoy en el nivel 7 paso al 8
    jr  nz, nivelOchoV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #6
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #8
            call    game_man_setNivel
            call    puertaCorrectaVertical
            call    entity_man_setPlayerPos
        jp  endV

    nivelOchoV:
    cp  #8 ;;Si estoy en el nivel 8 paso al 9
    jr  nz, nivelNueveV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #6
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #9
            call    game_man_setNivel
            call    puertaCorrectaVertical
            call    entity_man_setPlayerPos
        jp  endV

    nivelNueveV:
    cp  #9 ;;Si estoy en el nivel 9 paso al 10
    jr  nz, ultimoNivelV
        ;Compruebo que he eliminado a todos los enemigos del nivel para poder pasar al siguiente
        call    entity_man_getDefeatedEnemies
        cp      a, #6
        jp      nz, endV
            ; Reseteo las variables de spawn de enemigos y el contador de enemigos derrotados
            call game_man_resetEnemySpawn
            call entity_man_resetDefeatedEnemies
            ; Activo que se repinten los tiles
            ld      a, #1
            call    game_man_setBooleanTiles
            ; Cambio de sala
            ld      a, #10
            call    game_man_setNivel
            call    puertaCorrectaVertical
            call    entity_man_setPlayerPos
        jp  endV

    ultimoNivelV:
        ld  a, #6
        call game_man_setState
        call render_sys_win
endV:
ret

puertaCorrectaHorizontal::

    call entity_man_getArray

    ;Depending of e_x the player appears in different positions
    ld      a, e_x(ix)
    cp      #10
    jp      m, _puertaDrcha
    ld      a, #2   ;;ADJUST
    jr      _continue
    _puertaDrcha:
    ld      a, #76
    _continue:
    ld      b, e_y(ix) ;;ADJUST
ret

puertaCorrectaVertical::
    call entity_man_getArray

    ;Depending of e_y the player appears in different positions
    ld      a, e_y(ix)
    ;ub     #0
    cp #100
    jp      m, _puertaArriba
    ld      b, #0   ;;ADJUST
    jp      _continueV
    _puertaArriba:
    ld      b, #160
    _continueV:
    ld      a, e_x(ix) ;;ADJUST
ret

game_man_getState::
    ld  a, (state)
ret

game_man_setState::
    ld  (state), a
ret

game_man_getBooleanTiles::
    ld  a, (boolean_tiles)
ret

game_man_setBooleanTiles::
    ld  (boolean_tiles), a
ret

game_man_getNivel::
    ld  a, (nivel)
ret

game_man_setNivel::
    ld  (nivel), a
ret
