;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------

.module input_system
.include "cpctelera.h.s"
.include "entity_manager.h.s"
.include "cpctelera_functions.h.s"
.include "game_manager.h.s"
.include "physics_system.h.s"
.include "render_system.h.s"

.globl _sp_hair


cooldownTiro = 10

screen_w = 80
screen_h = 200

_cont: .db 0

input_sys_init::
    ld (_entity_array_ptr), hl
    ld a, #cooldownTiro
    ld (_cont), a
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Check if the key that plays the game is pressed
;; if pressed change game state
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
input_sys_menu::
    call cpct_scanKeyboard_f_asm
    call game_man_getState
    cp  #2
    jr   z, menu

        ld  hl,#Key_Esc
        call cpct_isKeyPressed_asm
        jr nz, init_if
        ld  hl,#Joy0_Fire1
        call cpct_isKeyPressed_asm
        jr z, end_if
        init_if:
            call game_man_init
            ld a, #2
            call game_man_setState
            call  cpct_waitVSYNC_asm
            call render_sys_menu
            jr end_if

menu:
    ld    hl,#Key_1
    call cpct_isKeyPressed_asm
    jr    nz, One_Pressed
    ld    hl,#Joy0_Up
    call cpct_isKeyPressed_asm
    jr    z, One_NotPressed
One_Pressed:
    cpctm_clearScreen_asm #0x00
    call game_man_init
    ld  a, #1
    call game_man_setState
    jr  end_if
One_NotPressed:
    ld    hl,#Key_2
    call cpct_isKeyPressed_asm
    jr    nz, Two_Pressed
    ld    hl,#Joy0_Left
    call cpct_isKeyPressed_asm
    jr    z, Two_NotPressed
Two_Pressed:
    ld  a, #4
    call game_man_setState
    call  cpct_waitVSYNC_asm
    call render_sys_control
    jr  end_if
Two_NotPressed:
    ld    hl,#Key_3
    call cpct_isKeyPressed_asm
    jr    nz, Three_Pressed
    ld    hl,#Joy0_Down
    call cpct_isKeyPressed_asm
    jr    z, Three_NotPressed
Three_Pressed:
    ld  a, #5
    call game_man_setState
    call  cpct_waitVSYNC_asm
    call render_sys_credit
Three_NotPressed:
end_if:
ret

input_sys_update::

    _entity_array_ptr = . + 2
    ld ix, #0x0000

    ld e_vx(ix), #0
    ld e_vy(ix), #0

    call cpct_scanKeyboard_f_asm


;;------------------------------------------------------------------------------------------
    ;; Input IA
    ld      e_ai_aim_x(ix), #0
    ld      hl, #Key_S              ;; HL holds Key_S
    call    cpct_isKeyPressed_asm   ;; Check if Key_S
    jr      z, S_NotPressed         ;; If (not pressed)
S_Pressed:
        ld      e_ai_aim_x(ix), #1  ;; Key_S is pressed
S_NotPressed:
;;------------------------------------------------------------------------------------------


    ld e, #0

    ld hl,#Key_O
    call cpct_isKeyPressed_asm
    jr nz, O_Pressed
    ld hl,#Joy0_Left
    call cpct_isKeyPressed_asm
    jr z, O_NotPressed
O_Pressed:
    ld e_vx(ix), #-1
    inc e
O_NotPressed:


    ld hl,#Key_P
    call cpct_isKeyPressed_asm
    jr nz, P_Pressed
    ld hl,#Joy0_Right
    call cpct_isKeyPressed_asm
    jr z, P_NotPressed
P_Pressed:
    ld e_vx(ix), #1
    inc e
P_NotPressed:


    ld hl,#Key_Q
    call cpct_isKeyPressed_asm
    jr nz, Q_Pressed
    ld hl,#Joy0_Up
    call cpct_isKeyPressed_asm
    jr z, Q_NotPressed
Q_Pressed:
    ld e_vy(ix), #-4
    inc e
Q_NotPressed:


    ld hl,#Key_A
    call cpct_isKeyPressed_asm
    jr nz, A_Pressed
    ld hl, #Joy0_Down
    call cpct_isKeyPressed_asm
    jr z, A_NotPressed
A_Pressed:
    ld e_vy(ix), #4
    inc e
A_NotPressed:
    dec e
    jp m, NotPressed
Pressed:
    ld a, e_vx(ix)
    ld e_dir_x(ix), a
    ld a, e_vy(ix)
    ld e_dir_y(ix), a
NotPressed:

ld  hl,#Key_Esc
call cpct_isKeyPressed_asm
jr nz, si_Esc
ld hl, #Joy0_Fire2
call cpct_isKeyPressed_asm
jr z, end_Esc
si_Esc:
    ld a, #2
    call game_man_setState
    ld a, #1
    call game_man_setBooleanTiles
    call  cpct_waitVSYNC_asm
    call render_sys_menu
end_Esc:



    ld a, (_cont)
    dec a
    cp #0
    jp nz, NoDisparo
Disparo:                                        ;if cooldown, player can shoot
                                                ;if not, cooldown dec
    ld hl,#Key_Space
    call cpct_isKeyPressed_asm                  ;if Space is pressed, player shoots
    jr nz, Spc_Pressed
    ld hl,#Joy0_Fire1
    call cpct_isKeyPressed_asm                  ;if Space is pressed, player shoots
    jr z, Spc_NotPressed
    Spc_Pressed:

        ld a, #cooldownTiro
        ld (_cont), a

        call calcnuevapos

        push hl
        push de
        call validBullet                            ;Determine if the bullet is goint to be created outside the screen
        pop de
        pop hl
        cp #1
        jr nz, Spc_NotPressed
        debug::
        push hl
        push de
        call ValidBulletTile
        pop de
        pop hl
        debug1::
        cp #1
        jr nz, Spc_NotPressed
            call entity_man_newBullet
            cp #-1
            jr z,Spc_NotPressed
        jr Disparoend
    Spc_NotPressed:
        ld a, #1
        ld (_cont), a
        jr Disparoend
NoDisparo:
    ld (_cont), a
Disparoend:

ret
;;
;;
;;
;;
;;
calcnuevapos::
   ld a, e_dir_y(ix)
            cp #0
            jr z, same_y
            cp #4
            jr z, down_y
        up_y:
            ld a, #-8
            ld e, a
            ld a, #-6
            add a, e_y(ix)
            jr end_y
        down_y:
            ld a, #8
            ld e, a
            ld a, #8
            add a, e_y(ix)
            jr end_y
        same_y:
            ld a, e_y(ix)
            ld e, #0
        end_y:
            ld d, a

        ;Calculate velocityX and PosX
            ld a, e_dir_x(ix)
            cp #0
            jr z, same_x
            cp #1
            jr z, right_x
        left_x:
            ld a, #-2
            ld l, a
            ld a, #-2
            add a, e_x(ix)
            jr end_x
        right_x:
            ld a, #2
            ld l,a
            ld a, #4
            add a, e_x(ix)
            jr end_x
        same_x:
            ld a, e_x(ix)
            ld l, #0
        end_x:
            ld h, a
ret