;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------

;;
;;  Physics System
;;
.module physicsSystem
.include "entity_manager.h.s"
.include "cpctelera.h.s"
.include "cpctelera_functions.h.s"
.include "game_manager.h.s"
.include "collision_system.h.s"

screen_w = 80
screen_h = 168
background = 0x0F
UI = 0x00
;   --- PHYSICS SYSTEM INIT
;   INPUT:
;           HL -> Pointer to entity array
;
physics_sys_init::
     ld (_entity_array_ptr), hl
ret

;;  --- PHYSICS SYSTEM ENTITY COLLISION
;;  Detects bounding box collision between any pair of entities
;;  INPUT:
;;          IX -> Pointer to current entity
;;          HL -> Pointer to other entity
;;  DESTROYS: AF, HL, DE
;;  RETURNS:
;;              A -> 0xFF if collision occurred
physics_sys_ent_collision::



;Compare X->
;enemy_y + enemy_h - player_y
    ld  a, e_y(iy)
    add a, e_h(iy)
    add a, e_vy(iy)
    sub e_y(ix)
    sub e_vy(ix)
    jr z, no_collision
    jp m, no_collision

;player_y + player_h - enemy_y
    ld  a, e_y(ix)
    add a, e_h(ix)
    add a, e_vy(ix)
    sub e_y(iy)
    sub e_vy(iy)
    jr z, no_collision
    jp m, no_collision

;enemy_x + enemy_w - player_x <= 0
    ld  a, e_x(iy)             ;a = enemy_x
    add a, e_w(iy)             ;a = enemy_x + enemy_w
    add a, e_vx(iy)
    sub e_x(ix)
    sub e_vx(ix)
    jr z, no_collision
    jp m, no_collision

;player_x + player_w - enemy_x <= 0
    ld  a, e_x(ix)
    add a, e_w(ix)
    add a, e_vx(ix)
    sub e_x(iy)
    sub e_vx(iy)
    jr z, no_collision
    jp m, no_collision

collision:
    ;saveSetBorder HW_RED

    ld b, #0
    ld a, e_ai_st(ix)
    call entity_man_detectType

    ld a, e_ai_st(iy)
    call entity_man_detectType

    call entity_man_updatePair

    ret
no_collision:

ret

ValidBulletTile::
;;comparo la X positiva
    push hl
    push de

    ld c, h
    ld b, d
    ld de, #CPCT_VMEM_START_ASM
    push bc
    call cpct_getScreenPtr_asm      ;video memory location
    ld a, (hl)
    cp #background
    pop bc
    pop de
    pop hl
    jr nz, noCreoxn
    ;;COMPARO OTRO pixel
    push hl
    push de

    ld a, #5
    add a, b
    ld b, a

    ld de, #CPCT_VMEM_START_ASM
    push bc
    call cpct_getScreenPtr_asm      ;video memory location
    ld a, (hl)
    cp #background
    pop bc
    pop de
    pop hl
    jr nz, noCreoxn

    ;;miro otra esquina
    ld a, #3
    add a, c
    ld de, #CPCT_VMEM_START_ASM
    push bc
    call cpct_getScreenPtr_asm      ;video memory location
    ld a, (hl)
    cp #background
    pop bc
    jr nz, noCreoxn

    ;ultimo
    ld a, #-4
    add a, b
    ld de, #CPCT_VMEM_START_ASM
    call cpct_getScreenPtr_asm      ;video memory location
    ld a, (hl)
    cp #background
    jr nz, noCreoxn
        ld a,#1
        ret
    noCreoxn:

    xor     a, a                    ;;reset A
    endComprob:
ret
;;  -- Tile Collision --
;;  the next 4 methods are the same
;;  INPUT:
;;  OUTPUT:
;;          DE -> color value of next position
collisionPosX::
    ld de, #CPCT_VMEM_START_ASM
    ld  a, e_x(ix)
    add a, e_vx(ix)
    add a, e_w(ix)
    dec a

    ld  c, a                        ;pointer to last column/first row pixel on the sprite
    ld  b, e_y(ix)                  ;position y
    push bc
    call cpct_getScreenPtr_asm      ;video memory location

    pop bc
    ld a, e_h(ix)
    add a, b
    dec a
    ld b, a

    ld d, (hl)
    push de
    ld de, #CPCT_VMEM_START_ASM
    call cpct_getScreenPtr_asm      ;video memory location
    pop de
    ld e, (hl)
ret
collisionNegX::
    ld de, #CPCT_VMEM_START_ASM
    ld  a, e_x(ix)
    add a, e_vx(ix)

    ld  c, a                        ;pointer to last column/first row pixel on the sprite
    ld  b, e_y(ix)                  ;position y
    push bc
    call cpct_getScreenPtr_asm      ;video memory location

    pop bc
    ld a, e_h(ix)
    add a, b
    dec a
    ld b, a

    ld d, (hl)
    push de
    ld de, #CPCT_VMEM_START_ASM
    call cpct_getScreenPtr_asm      ;video memory location
    pop de
    ld e, (hl)
ret

collisionPosY::
    ld de, #CPCT_VMEM_START_ASM
    ld  a, e_y(ix)
    add a, e_vy(ix)
    add a, e_h(ix)
    dec a

    ld  b, a                        ;pointer to last column/first row pixel on the sprite
    ld  c, e_x(ix)                  ;position x
    push bc
    call cpct_getScreenPtr_asm      ;video memory location

    pop bc
    ld a, e_w(ix)
    add a, c
    dec a
    ld c, a

    ld d, (hl)
    push de
    ld de, #CPCT_VMEM_START_ASM
    call cpct_getScreenPtr_asm      ;video memory location
    pop de
    ld e, (hl)
ret
collisionNegY::
    ld de, #CPCT_VMEM_START_ASM
    ld  a, e_y(ix)
    add a, e_vy(ix)

    ld  b, a                        ;pointer to last column/first row pixel on the sprite
    ld  c, e_x(ix)                  ;position y
    push bc
    call cpct_getScreenPtr_asm      ;video memory location

    pop bc
    ld a, e_w(ix)
    add a, c
    dec a
    ld c, a

    ld d, (hl)
    push de
    ld de, #CPCT_VMEM_START_ASM
    call cpct_getScreenPtr_asm      ;video memory location
    pop de
    ld e, (hl)
ret

;   --- PHYSICS SYSTEM UPDATE ---
;
physics_sys_update::
    _entity_array_ptr = . + 2
    ld ix, #0x0000
_update_loop:
    ld  a, e_w(ix)
    cp #e_w_invalid
    ret z

    ;; UPDATE X
    ld  a,#screen_w + 1
    sub e_w(ix)
    ld  c,a                 ;;C is the screen limit of the entity

    ld  a, e_x(ix)
    add a,e_vx(ix)
    cp  c                   ;if current pos+vel is > c
    jr nc, invalid_X
valid_X:
        push af
        ld a, e_vx(ix)
        xor #0
        jr z, noCollisionX
        jp m, negCollX
            call collisionPosX
            ld a,d
            cp #background
            jr nz,noCollisionX
            ld a, e
            cp #background
            jr nz,noCollisionX
            jr collisionX
        negCollX:
            call collisionNegX
            ld a,d
            cp #background
            jr nz,noCollisionX
            ld a, e
            cp #background
            jr nz,noCollisionX
            jr collisionX

        collisionX:
            pop af
            ld e_x(ix),a
            jr endif_X
        noCollisionX:
            pop af
            ld e_vx(ix), #0
    jr endif_X
invalid_X:                  ;change orientation of vel
    ld a, e_vx(ix)
    neg
    ld e_vx(ix),a

    ;========================================
    ; CAMBIAR ESTO DE SITIO, ACCEDER SOLO A LA POSICION DEL
    ; PLAYER Y CAMBIAR AL NIVEL DESEADO ANTES DE LA LLAMADA
    ; A GAME_MAN_SETNIVEL (1-2-3-4)
    ;=======================================

    ld      a, e_ai_st(ix)
    cp #e_ai_st_noAI
    jr      nz, otherEntX
        ; JUGADOR COLISIONA HORIZONTALMENTE

        call game_man_changeNivelHorizontal
        call entity_man_deleteAllBullets
        call entity_man_errasePowerUP
        jr endif_X
    otherEntX:
        ld e_vx(ix), #0
endif_X:

    ;; UPDATE Y
    ld  a,#screen_h + 1
    sub e_h(ix)
    ld  c,a                 ;;C is the screen limit of the entity

    ld  a, e_y(ix)
    add a,e_vy(ix)
    cp  c                   ;if current pos+vel is > c
    jr nc, invalid_Y
valid_Y:                    ;
    push af
    ld a, e_vy(ix)
    xor #0
    jr z, noCollisionY
    jp m, negCollY

        call collisionPosY
        ld a,d
        cp #background
        jr z,miroE
        cp #UI
        jr nz,noCollisionY
        miroE:
        ld a,e
        cp #background
        jr z,collisionY
        cp #UI
        jr z,collisionY
        jr noCollisionY
    negCollY:
        call collisionNegY
        ld a,d
        cp #background
        jr nz,noCollisionY
        ld a, e
        cp #background
        jr nz,noCollisionY
        jr collisionY
    collisionY:
        pop af
        ld e_y(ix),a
        jr endif_Y
    noCollisionY:
        ld e_vy(ix), #0
        pop af
    jr endif_Y
invalid_Y:                  ;change orientation of vel
    ld a, e_vy(ix)
    neg
    ld e_vy(ix),a

    ld      a, e_ai_st(ix)
    cp #e_ai_st_noAI
    jr      nz, otherEntY

        ; JUGADOR COLISIONA VERTICALMENTE
        call game_man_changeNivelVertical
        call entity_man_deleteAllBullets
        call entity_man_errasePowerUP
        jr endif_Y
    otherEntY:
        ld e_vy(ix), #0
endif_Y:

    ld a, e_ai_st(ix)
    cp #e_ai_st_bullet
    jr nz, paso
        ld a, e_vx(ix)
        add a, e_vy(ix)
        cp #0
        jr nz, paso
             call entity_man_dec_heart_IX
paso:
    ld de,#e_sizeof        ;; DE = sizeof(Entity_t)
    add ix, de             ;; IX Points to next Entity_t
jp _update_loop            ;; Continue updating




;;
;;INPUT H -> pos x   D -> pos y
;;      L -> vel x   E -> vel y
validBullet::

    ld  a,#screen_w + 1
    sub #3
    ld  c,a                 ;;C is the screen limit of the entity

    ld  a, h
    cp  c                   ;if current pos+vel is > c
    jr nc, invalidBullet_X
validBullet_X:                    ;
    ;;Se puede crear
    ld a, #1
    jr end_X
invalidBullet_X:
    ld a, #0
end_X:
    ex af, af'

    ld  a,#screen_h + 1
    sub #6
    ld  c,a                 ;;C is the screen limit of the entity

    ld  a, d
    cp  c                   ;if current pos+vel is > c
    jr nc, invalidBullet_Y
validBullet_Y:
    ex af, af'
    cp a, #1
    jr nz, invalidBullet_Y
    jr end_Y
invalidBullet_Y:
    ld a, #0
end_Y:

ret
