;;---------------------------------------- LICENSE NOTICE ----------------------------------------
;;
;;  This file is part of Angry Kitten: an Amstrad CPC Game made with CPCTelera
;;
;;     Copyright (C) 2019  Irene Clemente Aracil (@irenara), Araceli Lucía Rebollo Domínguez
;;      (@LuciaMew) and Jose Manuel Manresa Guirao (@jmmgMultimedia)
;;
;;
;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <https://www.gnu.org/licenses/>.
;;
;;------------------------------------------------------------------------------------------------

;;
;; RENDER SYSTEM
;;
.module renderSystem
.include "cpctelera_functions.h.s"
.include "entity_manager.h.s"
.include "assets/screenMenuz.h.s"
.include "assets/screenControlsz.h.s"
.include "assets/screenCreditsz.h.s"
.include "assets/screenGameoverz.h.s"
.include "assets/screenWinz.h.s"
.globl _pal_main
.globl _sp_heart

screen_start = 0xc000

render_sys_init::
    ld (_entity_array_ptr), hl
    ld (_first_ent), hl
    push hl

    ld c,#0
    call cpct_setVideoMode_asm
    ld hl, #_pal_main
    ld de, #16
    call cpct_setPalette_asm
    ;cpct_setBorder_asm HW_BLACK

    pop hl

ret

render_sys_menu::
    ld hl, # _screenMenuz_end                           ;decompress in video memory
    ld de, #0xffff
    call cpct_zx7b_decrunch_s_asm
ret
render_sys_control::
    ld hl, # _screenControlsz_end                       ;decompress in video memory
    ld de, #0xffff
    call cpct_zx7b_decrunch_s_asm
ret
render_sys_credit::
    ld hl, # _screenCreditsz_end                       ;decompress in video memory
    ld de, #0xffff
    call cpct_zx7b_decrunch_s_asm
ret
render_sys_gameover::
    ld hl, # _screenGameoverz_end                       ;decompress in video memory
    ld de, #0xffff
    call cpct_zx7b_decrunch_s_asm
ret
render_sys_win::
    ld hl, # _screenWinz_end                       ;decompress in video memory
    ld de, #0xffff
    call cpct_zx7b_decrunch_s_asm
ret
render_sys_update::
    call render_sys_entities
ret

render_sys_entities::
    _entity_array_ptr = . + 2
    ld ix, #0x0000

_update_loop:
    ld a, e_ai_st(ix)
    cp #e_ai_st_planta
    jr nz, nothing
    nextEntity:
        ld bc, #e_sizeof
        add ix, bc
    nothing:

    ld  a, e_w(ix)
    cp #e_w_invalid
    ret z

    ;ld      a, e_cmp_type(ix)      ;;
    ;cp      #e_t_invalid           ;; FUNCIONA :D
    ;jr      z, _nex_entity         ;;

    ;;Painting the background
    ld e, e_last_pos_l(ix)    ;;pointer to sprite
    ld d, e_last_pos_h(ix)
    ld a, #0x000f
    ld c, e_w(ix)               ;;X
    ld b, e_h(ix)               ;;Y
    push bc
    call cpct_drawSolidBox_asm

   ;;Calc. new position
    ld de, #screen_start
    ld c, e_x(ix)
    ld b, e_y(ix)
    call cpct_getScreenPtr_asm

    ;;save for next render
    ld e_last_pos_h(ix), h
    ld e_last_pos_l(ix), l

    ex de,hl
    pop bc

    ld h,e_sprite_ptr_h(ix)
    ld l,e_sprite_ptr_l(ix)
    call cpct_drawSprite_asm

_nex_entity:
    ld bc, #e_sizeof
    add ix, bc
    jr _update_loop



render_system_drawVida::
    _first_ent = . + 2
    ld ix, #0x0000
    ld a, e_vida(ix)
    ld de, #first_heart_pos
loopVida:
    cp #0
    jr z, endlooopVida
    ex af, af'
        ld hl, #_sp_heart
        ld bc, #0x0c04
        push de
            call cpct_drawSprite_asm
        pop de
        ld hl, #betweenHearts
        add hl, de
        ex  de, hl
    ex af, af'
    dec a
    jr loopVida
endlooopVida:
ret